# -*- coding: utf-8 -*-
from pip_services4_data.query import FilterParams
from eic_aichat_users.roles.data.UserRolesV1 import UserRolesV1
from eic_aichat_users.roles.persistence.IRolesPersistence import IRolesPersistence

ROLES = ['Role 1', 'Role 2', 'Role 3']


class RolesPersistenceFixture:
    def __init__(self, persistence: IRolesPersistence):
        assert persistence is not None
        self._persistence = persistence

    def test_get_and_set_roles(self):
        # Set roles
        roles = self._persistence.set(None, UserRolesV1('1', ROLES))
        assert len(roles.roles) == 3

        # Get by id
        roles = self._persistence.get_one_by_id(None, '1')
        assert roles is not None
        assert len(roles.roles) == 3

        # Filter by roles
        page = self._persistence.get_page_by_filter(
            None,
            FilterParams.from_tuples('roles', ['Role 1', 'Role X']),
            None
        )
        assert page is not None
        assert len(page.data) == 1



# -*- coding: utf-8 -*-
from eic_aichat_users.roles.persistence.RolesMemoryPersistence import RolesMemoryPersistence
from test.roles.persistence.RolesPersistenceFixture import RolesPersistenceFixture


class TestRolesMemoryPersistence:
    persistence: RolesMemoryPersistence
    fixture: RolesPersistenceFixture

    def setup_method(self):
        self.persistence = RolesMemoryPersistence()
        self.fixture = RolesPersistenceFixture(self.persistence)

        self.persistence.open(None)
        self.persistence.clear(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_get_and_set_roles(self):
        self.fixture.test_get_and_set_roles()



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data.UserRolesV1 import UserRolesV1


class IRolesPersistence(ABC):
    @abstractmethod
    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        pass

    @abstractmethod
    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[UserRolesV1]:
        pass

    @abstractmethod
    def set(self, context: Optional[IContext], item: UserRolesV1) -> UserRolesV1:
        pass



# -*- coding: utf-8 -*-
from typing import Optional
from datetime import datetime

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from ..data.UserRolesV1 import UserRolesV1
from .IRolesPersistence import IRolesPersistence


class RolesMongoDbPersistence(IdentifiableMongoDbPersistence, IRolesPersistence):
    def __init__(self):
        super().__init__('user_roles')

    def _convert_to_public(self, value: any) -> any:
        if value is None:
            return None
        
        user_roles = UserRolesV1(
            id=value.get('_id'),
            roles=value.get('roles')
        )
        user_roles.update_time = value.get('update_time', datetime.now())
        return user_roles

    def __compose_filter(self, filter_params: FilterParams):
        filter_params = filter_params or FilterParams()
        criteria = []

        id = filter_params.get_as_nullable_string('id')
        if id is not None:
            criteria.append({'_id': id})

        ids = filter_params.get_as_object('ids')
        if isinstance(ids, str):
            ids = ids.split(',')
        if isinstance(ids, list):
            criteria.append({'_id': {'$in': ids}})

        except_ids = filter_params.get_as_object('except_ids')
        if isinstance(except_ids, str):
            except_ids = except_ids.split(',')
        if isinstance(except_ids, list):
            criteria.append({'_id': {'$nin': except_ids}})

        roles = filter_params.get_as_object('roles')
        if isinstance(roles, str):
            roles = roles.split(',')
        if isinstance(roles, list):
            criteria.append({'roles': {'$in': roles}})

        except_roles = filter_params.get_as_object('except_roles')
        if isinstance(except_roles, str):
            except_roles = except_roles.split(',')
        if isinstance(except_roles, list):
            criteria.append({'roles': {'$nin': except_roles}})

        return {'$and': criteria} if criteria else None

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        return super().get_page_by_filter(context, self.__compose_filter(filter_params), paging)

    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[UserRolesV1]:
        return super().get_one_by_id(context, id)

    def set(self, context: Optional[IContext], item: UserRolesV1) -> UserRolesV1:
        item.update_time = datetime.utcnow()
        return super().set(context, item)



# -*- coding: utf-8 -*-
from pip_services4_data.validate import ObjectSchema
from pip_services4_commons.convert import TypeCode


class SettingsSectionV1Schema(ObjectSchema):
    def __init__(self):
        super().__init__()
        self.with_required_property('id', TypeCode.String)
        self.with_optional_property('parameters', TypeCode.Map)
        self.with_optional_property('update_time', TypeCode.DateTime)



# -*- coding: utf-8 -*-
import datetime

from pip_services4_components.context import Context
from pip_services4_data.query import FilterParams, PagingParams

from eic_aichat_users.activities.data import PartyActivityV1, ReferenceV1


class ActivitiesPersistenceFixture:
    _persistence = None

    def __init__(self, persistence):
        self._persistence = persistence

    def test_crud_operations(self):
        # Create activity
        activity = PartyActivityV1(
            id='1',
            time=datetime.datetime.now(),
            type='test',
            party=ReferenceV1(id='1', type='party'),
            ref_item=ReferenceV1(id='1', type='item'),
            ref_parents=[],
            ref_party=None,
            details={}
        )

        created_activity = self._persistence.create(Context.from_trace_id('123'), activity)
        assert created_activity is not None
        # assert created_activity.id == activity.id
        assert created_activity.type == activity.type
        assert created_activity.party.id == activity.party.id

        # Get activity by id
        result = self._persistence.get_one_by_id(Context.from_trace_id('123'), created_activity.id)
        assert result is not None
        assert result.id == activity.id
        assert result.type == activity.type
        assert result.party.id == activity.party.id

        # Delete activity
        self._persistence.delete_by_id(Context.from_trace_id('123'), created_activity.id)

        # Try to get deleted activity
        result = self._persistence.get_one_by_id(Context.from_trace_id('123'), created_activity.id)
        assert result is None

    def test_get_with_filter(self):
        # Create activities
        activity1 = PartyActivityV1(
            id='1',
            time=datetime.datetime.now(),
            type='test',
            party=ReferenceV1(id='1', type='party'),
            ref_item=ReferenceV1(id='1', type='item'),
            ref_parents=[],
            ref_party=None,
            details={}
        )
        activity2 = PartyActivityV1(
            id='2',
            time=datetime.datetime.now(),
            type='test',
            party=ReferenceV1(id='1', type='party'),
            ref_item=ReferenceV1(id='2', type='item'),
            ref_parents=[],
            ref_party=None,
            details={}
        )
        activity3 = PartyActivityV1(
            id='3',
            time=datetime.datetime.now(),
            type='test',
            party=ReferenceV1(id='2', type='party'),
            ref_item=ReferenceV1(id='2', type='item'),
            ref_parents=[],
            ref_party=None,
            details={}
        )

        self._persistence.create(Context.from_trace_id('123'), activity1)
        self._persistence.create(Context.from_trace_id('123'), activity2)
        self._persistence.create(Context.from_trace_id('123'), activity3)

        # Get with filters
        page = self._persistence.get_page_by_filter(
            Context.from_trace_id('123'),
            FilterParams.from_tuples('party_id', '1'),
            PagingParams()
        )
        assert len(page.data) == 2

        page = self._persistence.get_page_by_filter(
            Context.from_trace_id('123'),
            FilterParams.from_tuples('ref_item_id', '2'),
            PagingParams()
        )
        assert len(page.data) == 2

        page = self._persistence.get_page_by_filter(
            Context.from_trace_id('123'),
            FilterParams.from_tuples('type', 'test'),
            PagingParams()
        )
        assert len(page.data) == 3 


# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data import GroupMembershipV1


class IGroupMembershipsPersistence(ABC):

    @abstractmethod
    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                            paging: PagingParams) -> DataPage:
        pass

    @abstractmethod
    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[GroupMembershipV1]:
        pass

    @abstractmethod
    def create(self, context: Optional[IContext], item: GroupMembershipV1) -> GroupMembershipV1:
        pass

    @abstractmethod
    def update(self, context: Optional[IContext], item: GroupMembershipV1) -> GroupMembershipV1:
        pass

    @abstractmethod
    def delete_by_id(self, context: Optional[IContext], id: str) -> Optional[GroupMembershipV1]:
        pass

    @abstractmethod
    def delete_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        pass



# -*- coding: utf-8 -*-
import json
import bottle
import asyncio

from typing import List
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_commons.errors import BadRequestException
from pip_services4_data.query import FilterParams
from pip_services4_components.context import Context

from eic_aichat_users.groupmemberships.data.GroupMembershipV1 import GroupMembershipV1
from eic_aichat_users.groupmemberships.logic.IGroupMembershipsService import IGroupMembershipsService
from eic_aichat_users.groups.data.GroupV1 import GroupV1
from eic_aichat_users.groups.logic.IGroupsService import IGroupsService
from eic_aichat_users.partialfacade.operations.version1.Authorize import AuthorizerV1


# TODO
class GroupsOperation(RestOperations):
    def __init__(self):
        super().__init__()
        self._group: IGroupsService = None
        self._group_memberships: IGroupMembershipsService = None
        self._dependency_resolver.put('groups', Descriptor('aichatusers-groups', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put('groupmemberships', Descriptor('aichatusers-groupmemberships', 'service', '*', '*', '1.0'))

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._group = self._dependency_resolver.get_one_required('groups')
        self._group_memberships = self._dependency_resolver.get_one_required('groupmemberships')

    def get_groups(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id

        filter_params = self._get_filter_params()
        paging = self._get_paging_params()

        filter_params.set_as_object('owner_id', user_id)
        try:
            page = self._group.get_groups(context, filter_params, paging)
            return self._send_result(page)
        except Exception as err:
            return self._send_error(err)

    def get_group_by_id(self, id: str):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            res = self._group.get_group_by_id(context, id)
            if res is None or res.id == "":
                raise BadRequestException('Group ' + id + ' not found')

            if res.owner_id != user_id:
                raise BadRequestException('Group ' + id + ' is not owned by user ' + user_id)
            
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def create_group(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id

        data = bottle.request.json
        group_data = data if isinstance(data, dict) else json.loads(data or '{}')
        group = GroupV1(**group_data)

        group.owner_id = user_id

        try:
            res = self._group.create_group(context, group)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def set_group(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id

        data = bottle.request.json
        group_data = data if isinstance(data, dict) else json.loads(data or '{}')
        group = GroupV1(**group_data)
        group.owner_id = user_id

        try:
            res = self._group.set_group(context, group)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def update_group(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id

        data = bottle.request.json
        group_data = data if isinstance(data, dict) else json.loads(data or '{}')
        group = GroupV1(**group_data)
        group.owner_id = user_id

        try:
            old_group = self._group.get_group_by_id(context, group.id)
            if old_group is None or old_group.id == "":
                raise BadRequestException('Group ' + id + ' not found')

            if old_group.owner_id != user_id:
                raise BadRequestException('Group ' + id + ' is not owned by user ' + user_id)
            
            res = self._group.update_group(context, group)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        

    def delete_group_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            old_group = self._group.get_group_by_id(context, id)
            if old_group is None or old_group.id == "":
                raise BadRequestException('Group ' + id + ' not found')

            if old_group.owner_id != user_id:
                raise BadRequestException('Group ' + id + ' is not owned by user ' + user_id)
            
            res = self._group.delete_group_by_id(context, id)

            page = self._group_memberships.get_memberships(context, FilterParams().set_as_object('group_id', id), None)
            for membership in page.data:
                self._group_memberships.delete_membership_by_id(context, membership.id)

            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def delete_group_by_filter(self):
        context = Context.from_trace_id(self._get_trace_id())
        filter_params = self._get_filter_params()
        try:
            res = self._group.delete_groups_by_filter(context, filter_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def register_routes(self, controller: RestController, auth: AuthorizerV1):
        controller.register_route_with_auth('get', '/users/groups', None, auth.signed(), lambda: self.get_groups())
        
        controller.register_route_with_auth('get', '/users/:id/groups', None, auth.signed(), lambda id: self.get_group_by_id(id))

        controller.register_route_with_auth('post', '/users/groups', None, auth.signed(), lambda: self.create_group())

        # controller.register_route_with_auth('post', '/users/groups/set', None, auth.signed(), lambda: self.set_group())

        controller.register_route_with_auth('put', '/users/groups', None, auth.signed(), lambda: self.update_group())

        controller.register_route_with_auth('delete', '/users/:id/groups', None, auth.signed(), lambda id: self.delete_group_by_id(id))

        controller.register_route_with_auth('delete', '/users/groups', None, auth.admin(), lambda: self.delete_group_by_filter())
        
    


# -*- coding: utf-8 -*-
from eic_aichat_users.accounts.persistence.AccountsMemoryPersistence import AccountsMemoryPersistence
from test.accounts.persistence.AccountsPersistenceFixture import AccountsPersistenceFixture


class TestAccountsMemoryPersistence:
    persistence: AccountsMemoryPersistence
    fixture: AccountsPersistenceFixture

    def setup_method(self):
        self.persistence = AccountsMemoryPersistence()
        self.fixture = AccountsPersistenceFixture(self.persistence)

        self.persistence.open(None)
        self.persistence.clear(None)

    def teardown_method(self):
        self.persistence.close(None)

    def test_crud_operations(self):
        self.fixture.test_crud_operations()

    def test_get_with_filter(self):
        self.fixture.test_get_with_filter()



# -*- coding: utf-8 -*-
from pip_services4_data.validate import ObjectSchema
from pip_services4_commons.convert import TypeCode


class SessionV1Schema(ObjectSchema):
    def __init__(self):
        super().__init__()

        self.with_optional_property('id', TypeCode.String)
        self.with_required_property('user_id', TypeCode.String)
        self.with_required_property('user_name', TypeCode.String)
        self.with_optional_property('active', TypeCode.Boolean)
        self.with_optional_property('open_time', None)
        self.with_optional_property('close_time', None)
        self.with_optional_property('request_time', None)
        self.with_optional_property('address', TypeCode.String)
        self.with_optional_property('client', TypeCode.String)
        self.with_optional_property('user', None)
        self.with_optional_property('data', None)



# -*- coding: utf-8 -*-

__all__ = ['AccountV1', 'AccountV1Schema', 'AccountActivityTypeV1']

from .AccountV1 import AccountV1
from .AccountV1Schema import AccountV1Schema
from .AccountActivityTypeV1 import AccountActivityTypeV1



# -*- coding: utf-8 -*-

__all__ = [
    'IGroupMembershipsPersistence', 'GroupMembershipsMemoryPersistence', 'GroupMembershipsMongoDbPersistence'
]

from .IGroupMembershipsPersistence import IGroupMembershipsPersistence
from .GroupMembershipsMemoryPersistence import GroupMembershipsMemoryPersistence
from .GroupMembershipsMongoDbPersistence import GroupMembershipsMongoDbPersistence



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Optional, Any

from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, FilterParams, PagingParams
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from eic_aichat_users.groups.data.GroupV1 import GroupV1
from eic_aichat_users.groups.persistence.IGroupsPersistence import IGroupsPersistence

class GroupsMongoDbPersistence(IdentifiableMongoDbPersistence, IGroupsPersistence):

    def __init__(self):
        super().__init__('groups')
        self._max_page_size = 1000

        self._ensure_index({'owner_id': 1})

    def _convert_to_public(self, value: Any) -> Optional[GroupV1]:
        if value is None:
            return None

        return GroupV1(
            id=value.get('_id'),
            title=value.get('title'),
            active_since=value.get('active_since'),
            owner_id=value.get('owner_id'),
            group_active=value.get('group_active'),
        )
    
    def __compose_filter(self, filter_params: FilterParams) -> Any:
        filter_params = filter_params or FilterParams()
        criteria = []

        id = filter_params.get_as_nullable_string('id')
        if id:
            criteria.append({'_id': id})

        ids = filter_params.get_as_object('ids')
        if isinstance(ids, str):
            ids = ids.split(',')
        if isinstance(ids, list):
            criteria.append({'_id': {'$in': ids}})

        owner_id = filter_params.get_as_nullable_string('owner_id')
        if owner_id:
            criteria.append({'owner_id': owner_id})

        not_in_owner_ids = filter_params.get_as_object('not_in_owner_ids')
        if isinstance(not_in_owner_ids, str):
            not_in_owner_ids = not_in_owner_ids.split(',')
        if isinstance(not_in_owner_ids, list):
            criteria.append({'owner_id': {'$nin': not_in_owner_ids}})

        title = filter_params.get_as_nullable_string('title')
        if title:
            criteria.append({'title': title})

        active_since = filter_params.get_as_nullable_datetime('active_since')
        if active_since:
            criteria.append({'active_since': active_since})

        group_active = filter_params.get_as_nullable_boolean('group_active')
        if group_active is not None:
            criteria.append({'group_active': group_active})

        return {'$and': criteria} if criteria else {}
    
    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams, sort: Any = None, select: Any = None) -> DataPage:
        return super().get_page_by_filter(
            context,
            self.__compose_filter(filter_params),
            paging,
            sort,
            select
        )

    def delete_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        return super().delete_by_filter(context, self.__compose_filter(filter_params))


# -*- coding: utf-8 -*-
from pip_services4_commons.convert import TypeCode
from pip_services4_data.validate import ObjectSchema


class GroupV1Schema(ObjectSchema):
    def __init__(self):
        super().__init__()

        self.with_optional_property('id', TypeCode.String)
        self.with_optional_property('title', TypeCode.String)
        self.with_optional_property('active_since', TypeCode.DateTime)
        self.with_required_property('owner_id', TypeCode.String)
        self.with_required_property('group_active', TypeCode.Boolean)
        self.with_required_property('description', TypeCode.String)
        self.with_required_property('member_count', TypeCode.Integer)



# -*- coding: utf-8 -*-
from typing import Any, Optional
from datetime import datetime

from pip_services4_data.data import IStringIdentifiable
from pip_services4_data.keys import IdGenerator


class UserPasswordV1(IStringIdentifiable):
    def __init__(
            self,
            id: Optional[str] = None,
            password: Optional[str] = None,
    ):
        # Identification 
        self.id: str = id or IdGenerator.next_long()
        self.password: str = password

        # Password management
        self.change_time: Optional[datetime] = None
        self.locked: bool = False
        self.lock_time: Optional[datetime] = None
        self.fail_count: int = 0
        self.fail_time: Optional[datetime] = None
        self.rec_code: Optional[str] = None
        self.rec_expire_time: Optional[datetime] = None

        # Custom fields
        self.custom_hdr: Optional[Any] = None
        self.custom_dat: Optional[Any] = None

    def to_dict(self):
        return {
            "id": self.id,
            "password": self.password,
            "change_time": self.change_time.isoformat() if isinstance(self.change_time, datetime) else self.change_time,
            "locked": self.locked,
            "lock_time":self.lock_time.isoformat() if isinstance(self.lock_time, datetime) else self.lock_time,
            "fail_count": self.fail_count,
            "fail_time": self.fail_time.isoformat() if isinstance(self.fail_time, datetime) else self.fail_time,
            "rec_code": self.rec_code,
            "rec_expire_time": self.rec_expire_time.isoformat() if isinstance(self.rec_expire_time, datetime) else self.rec_expire_time,
            "custom_hdr": self.custom_hdr,
            "custom_dat": self.custom_dat
        }
    



# -*- coding: utf-8 -*-

__all__ = [
    'IGroupsPersistence', 'GroupsMongoDbPersistence', 'GroupsMemoryPersistence'
]

from .IGroupsPersistence import IGroupsPersistence
from .GroupsMongoDbPersistence import GroupsMongoDbPersistence
from .GroupsMemoryPersistence import GroupsMemoryPersistence



# -*- coding: utf-8 -*-
from typing import Optional, Callable, Any
from datetime import datetime

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_persistence.persistence import IdentifiableMemoryPersistence

from ..data.UserRolesV1 import UserRolesV1
from .IRolesPersistence import IRolesPersistence


class RolesMemoryPersistence(IdentifiableMemoryPersistence, IRolesPersistence):
    def __init__(self):
        super().__init__()

    def __contains_any(self, array1, array2):
        if not array1 or not array2:
            return False
        return any(item in array2 for item in array1)

    def __compose_filter(self, filter_params: FilterParams) -> Callable[[UserRolesV1], bool]:
        filter_params = filter_params or FilterParams()

        id = filter_params.get_as_nullable_string('id')
        ids = filter_params.get_as_object('ids')
        except_ids = filter_params.get_as_object('except_ids')
        roles = filter_params.get_as_object('roles')
        except_roles = filter_params.get_as_object('except_roles')

        if isinstance(ids, str):
            ids = ids.split(',')
        if not isinstance(ids, list):
            ids = None

        if isinstance(except_ids, str):
            except_ids = except_ids.split(',')
        if not isinstance(except_ids, list):
            except_ids = None

        if isinstance(roles, str):
            roles = roles.split(',')
        if not isinstance(roles, list):
            roles = None

        if isinstance(except_roles, str):
            except_roles = except_roles.split(',')
        if not isinstance(except_roles, list):
            except_roles = None

        def filter_func(item: UserRolesV1) -> bool:
            if id and item.id != id:
                return False
            if ids and item.id not in ids:
                return False
            if except_ids and item.id in except_ids:
                return False
            if roles and not self.__contains_any(roles, item.roles):
                return False
            if except_roles and self.__contains_any(except_roles, item.roles):
                return False
            return True

        return filter_func

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        return super().get_page_by_filter(context, self.__compose_filter(filter_params), paging)

    def get_one_by_id(self, context: Optional[IContext], id: str) -> Optional[UserRolesV1]:
        return super().get_one_by_id(context, id)

    def set(self, context: Optional[IContext], item: UserRolesV1) -> UserRolesV1:
        item.update_time = datetime.utcnow()
        return super().set(context, item)



# -*- coding: utf-8 -*-
from typing import Optional
from pip_services4_components.context import IContext
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from .IPasswordsPersistence import IPasswordsPersistence
from ..data.version1.UserPasswordV1 import UserPasswordV1


class PasswordsMongoDbPersistence(IdentifiableMongoDbPersistence, IPasswordsPersistence):
    
    def __init__(self):
        super().__init__('passwords')
        self._max_page_size = 1000

    def _convert_to_public(self, value: any) -> any:
        if value is None:
            return None
        
        user_password = UserPasswordV1(
            id=value.get('_id'),
            password=value.get('password')
        )
        user_password.change_time = value.get('change_time')
        user_password.locked = value.get('locked', False)
        user_password.lock_time = value.get('lock_time')
        user_password.fail_count = value.get('fail_count', 0)
        user_password.fail_time = value.get('fail_time')
        user_password.rec_code = value.get('rec_code')
        user_password.rec_expire_time = value.get('rec_expire_time')
        user_password.custom_hdr = value.get('custom_hdr')
        user_password.custom_dat = value.get('custom_dat')
        return user_password


# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Optional
from pip_services4_data.data import IStringIdentifiable
from pip_services4_components.config import ConfigParams


class SettingsSectionV1(IStringIdentifiable):
    def __init__(self, id: str = None, parameters: Optional[ConfigParams] = None):
        self.id = id
        self.parameters = ConfigParams.from_value(parameters or {})
        self.update_time = datetime.now()

    def to_dict(self):
        return {
            "id": self.id,
            "parameters": self.parameters.to_dict() if hasattr(self.parameters, "to_dict") else self.parameters,
            "update_time": self.update_time.isoformat() if isinstance(self.update_time, datetime) else self.update_time
        }
    



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Dict, List, Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, FilterParams, PagingParams

from ..data import AccountV1


class IAccountsService(ABC):

    @abstractmethod
    def get_accounts(self, context: Optional[IContext], filter_params: FilterParams, paging: PagingParams) -> DataPage:
        raise NotImplementedError()

    @abstractmethod
    def get_account_by_id(self, context: Optional[IContext], account_id: str) -> AccountV1:
        raise NotImplementedError()

    @abstractmethod
    def get_account_by_login(self, context: Optional[IContext], login: str) -> AccountV1:
        raise NotImplementedError()

    @abstractmethod
    def get_account_by_id_or_login(self, context: Optional[IContext], id_or_login: str) -> AccountV1:
        raise NotImplementedError()

    @abstractmethod
    def create_account(self, context: Optional[IContext], account: AccountV1) -> AccountV1:
        raise NotImplementedError()

    @abstractmethod
    def update_account(self, context: Optional[IContext], account: AccountV1) -> AccountV1:
        raise NotImplementedError()

    @abstractmethod
    def delete_account_by_id(self, context: Optional[IContext], account_id: str) -> AccountV1:
        raise NotImplementedError()

    @abstractmethod
    def drop_account_by_id(self, context: Optional[IContext], account_id: str) -> AccountV1:
        raise NotImplementedError()
    
    @abstractmethod
    def get_map_by_ids(self, context: Optional[IContext], account_ids: List[str]) -> Dict[str, AccountV1]:
        raise NotImplementedError()



# -*- coding: utf-8 -*-
from typing import Optional, Callable, Any

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_persistence.persistence import IdentifiableMemoryPersistence

from ..data import GroupMembershipV1
from .IGroupMembershipsPersistence import IGroupMembershipsPersistence


class GroupMembershipsMemoryPersistence(IdentifiableMemoryPersistence, IGroupMembershipsPersistence):

    def __init__(self):
        super().__init__()
        self._max_page_size = 100

    def __compose_filter(self, filter_params: Optional[FilterParams]) -> Callable[[GroupMembershipV1], bool]:
        filter_params = filter_params or FilterParams()

        id = filter_params.get_as_nullable_string('id')
        profile_id = filter_params.get_as_nullable_string('profile_id')
        group_id = filter_params.get_as_nullable_string('group_id')
        active = filter_params.get_as_nullable_boolean('active')
        offline_or_online_id = filter_params.get_as_nullable_string('offline_or_onlie_id')

        def to_list(value: Any) -> Optional[list]:
            if isinstance(value, str):
                return value.split(',')
            if isinstance(value, list):
                return value
            return None

        ids = to_list(filter_params.get_as_object('ids'))
        profile_ids = to_list(filter_params.get_as_object('profile_ids'))
        group_ids = to_list(filter_params.get_as_object('group_ids'))
        offline_or_online_ids = to_list(filter_params.get_as_object('offline_or_onlie_id'))

        def predicate(item: GroupMembershipV1) -> bool:
            if id and item.id != id:
                return False
            if ids and item.id not in ids:
                return False
            if profile_ids and item.profile_id not in profile_ids:
                return False
            if group_ids and item.group_id not in group_ids:
                return False
            if profile_id and item.profile_id != profile_id:
                return False
            if group_id and item.group_id != group_id:
                return False
            if active is not None and item.active != active:
                return False
            if offline_or_online_id and item.profile_id != offline_or_online_id and item.offline_id != offline_or_online_id:
                return False
            if offline_or_online_ids and item.profile_id not in offline_or_online_ids and item.offline_id not in offline_or_online_ids:
                return False
            return True

        return predicate

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        return super().get_page_by_filter(
            context,
            self.__compose_filter(filter_params),
            paging,
            None,
            None
        )

    def delete_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        return super().delete_by_filter(context, self.__compose_filter(filter_params))

    def create(self, context: Optional[IContext], item: GroupMembershipV1) -> GroupMembershipV1:
        self._check_if_exists(context, item)
        return super().create(context, item)

    def update(self, context: Optional[IContext], item: GroupMembershipV1) -> GroupMembershipV1:
        self._check_if_exists(context, item)
        return super().update(context, item)

    def _check_if_exists(self, context: Optional[IContext], item: GroupMembershipV1) -> None:
        page = self.get_page_by_filter(
            context,
            FilterParams.from_tuples(
                'profile_id', item.profile_id,
                'group_id', item.group_id
            ),
            None
        )

        if not item.id and page.data:
            item.id = page.data[0].id

        if len(page.data) > 1 or (page.data and page.data[0].id != item.id):
            raise Exception("Group membership already exists")



# -*- coding: utf-8 -*-
import json
import bottle
from typing import List

from pip_services4_commons.convert import TypeCode
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_data.validate import ObjectSchema
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_components.context import Context

from eic_aichat_users.partialfacade.operations.version1.Authorize import AuthorizerV1
from eic_aichat_users.roles.logic.IRolesService import IRolesService
from eic_aichat_users.roles.data import UserRolesV1


class RolesOperations(RestOperations):
    def __init__(self):
        super().__init__()
        self._roles_service: IRolesService = None
        self._dependency_resolver.put("roles-service", Descriptor('aichatusers-roles', 'service', '*', '*', '1.0'))

    def configure(self, config):
        super().configure(config)

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._roles_service = self._dependency_resolver.get_one_required("roles-service")

    def get_roles_by_filter(self):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------get_roles_by_filter() invoked")

        filter_params = self._get_filter_params()
        paging_params = self._get_paging_params()
        try:
            res = self._roles_service.get_roles_by_filter(context, filter_params, paging_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def get_roles_by_id(self, user_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------get_roles_by_id() invoked")

        try:
            res = self._roles_service.get_roles_by_id(context, user_id)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def set_roles(self, user_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------set_roles() invoked")

        data = bottle.request.json or []
        try:
            res = self._roles_service.set_roles(context, user_id, data)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def grant_roles(self, user_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------grant_roles() invoked")

        data = bottle.request.json or []
        try:
            res = self._roles_service.grant_roles(context, user_id, data)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def revoke_roles(self, user_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------revoke_roles() invoked")

        data = bottle.request.json or []
        try:
            res = self._roles_service.revoke_roles(context, user_id, data)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def authorize(self, user_id):
        context = Context.from_trace_id(self._get_trace_id())
        self._logger.info(context, "----------authorize() invoked")

        data = bottle.request.json or []
        try:
            res = self._roles_service.authorize(context, user_id, data)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)

    def register_routes(self, controller: RestController, auth: AuthorizerV1):
        # controller.register_route('get', '/roles', None, self.get_roles_by_filter)

        controller.register_route_with_auth('get', '/users/<user_id>/roles', ObjectSchema(True)
                                  .with_required_property("user_id", TypeCode.String), auth.owner_or_admin(), self.get_roles_by_id)

        # controller.register_route('post', '/roles/<user_id>/set', ObjectSchema(True)
        #                           .with_required_property("user_id", TypeCode.String)
        #                           .with_required_property("body", TypeCode.Array),
        #                           self.set_roles)

        controller.register_route_with_auth('post', '/users/<user_id>/roles/grant', ObjectSchema(True)
                                  .with_required_property("user_id", TypeCode.String)
                                  .with_required_property("body", TypeCode.Array), auth.admin(), self.grant_roles)

        controller.register_route_with_auth('post', '/users/<user_id>/roles/revoke', ObjectSchema(True)
                                  .with_required_property("user_id", TypeCode.String)
                                  .with_required_property("body", TypeCode.Array), auth.admin(), self.revoke_roles)

        # controller.register_route('post', '/roles/<user_id>/authorize', ObjectSchema(True)
        #                           .with_required_property("user_id", TypeCode.String)
        #                           .with_required_property("body", TypeCode.Array),
        #                           self.authorize)



# -*- coding: utf-8 -*-
from typing import Optional, Any

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_mongodb.persistence import IdentifiableMongoDbPersistence

from ..data import GroupMembershipV1
from .IGroupMembershipsPersistence import IGroupMembershipsPersistence


class GroupMembershipsMongoDbPersistence(IdentifiableMongoDbPersistence, IGroupMembershipsPersistence):

    def __init__(self):
        super().__init__('group_memberships')
        self._max_page_size = 100

        self._ensure_index({ "profile_id": 1 })
        self._ensure_index({ "group_id": 1 })
        self._ensure_index({ "group_id": 1,  "profile_id": 1}, { 'unique': True })

    def _compose_filter(self, filter_params: Optional[FilterParams]) -> Any:
        filter_params = filter_params or FilterParams()
        criteria = []

        def to_list(value):
            if isinstance(value, str):
                return value.split(',')
            elif isinstance(value, list):
                return value
            return None

        id = filter_params.get_as_nullable_string('id')
        if id:
            criteria.append({ '_id': id })

        ids = to_list(filter_params.get_as_object('ids'))
        if ids:
            criteria.append({ '_id': { '$in': ids } })

        profile_ids = to_list(filter_params.get_as_object('profile_ids'))
        if profile_ids:
            criteria.append({ 'profile_id': { '$in': profile_ids } })

        profile_id = filter_params.get_as_nullable_string('profile_id')
        if profile_id:
            criteria.append({ 'profile_id': profile_id })

        offline_id = filter_params.get_as_nullable_string('offline_id')
        if offline_id:
            criteria.append({ 'offline_id': offline_id })

        group_id = filter_params.get_as_nullable_string('group_id')
        if group_id:
            criteria.append({ 'group_id': group_id })

        active = filter_params.get_as_nullable_boolean('active')
        if active is not None:
            criteria.append({ 'active': active })

        attempts = filter_params.get_as_nullable_integer('attempts')
        if attempts is not None:
            criteria.append({ 'attempts': attempts })

        return { '$and': criteria } if criteria else {}

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                                 paging: PagingParams) -> DataPage:
        return super().get_page_by_filter(
            context,
            self._compose_filter(filter_params),
            paging,
            None,
            None
        )

    def delete_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        super().delete_by_filter(context, self._compose_filter(filter_params))

    def create(self, context: Optional[IContext], item: GroupMembershipV1) -> GroupMembershipV1:
        return super().create(context, item)



# -*- coding: utf-8 -*-
from typing import Optional, Callable, Any

from pip_services4_components.context import IContext
from pip_services4_data.query import DataPage, FilterParams, PagingParams
from pip_services4_persistence.persistence import IdentifiableMemoryPersistence

from eic_aichat_users.groups.data.GroupV1 import GroupV1
from eic_aichat_users.groups.persistence.IGroupsPersistence import IGroupsPersistence



class GroupsMemoryPersistence(IdentifiableMemoryPersistence, IGroupsPersistence):

    def __init__(self):
        super().__init__()
        self._max_page_size = 100

    def __compose_filter(self, filter_params: Optional[FilterParams]) -> Callable[[GroupV1], bool]:
        filter_params = filter_params or FilterParams()

        id = filter_params.get_as_nullable_string('id')
        owner_id = filter_params.get_as_nullable_string('orowner_idg_id')
        title = filter_params.get_as_nullable_string('title')
        active_since = filter_params.get_as_nullable_datetime('active_since')
        group_active = filter_params.get_as_nullable_boolean('group_active')

        ids = filter_params.get_as_object('ids')
        if isinstance(ids, str):
            ids = ids.split(',')
        if not isinstance(ids, list):
            ids = None

        profile_ids = filter_params.get_as_object('profile_ids')
        if isinstance(profile_ids, str):
            profile_ids = profile_ids.split(',')
        if not isinstance(profile_ids, list):
            profile_ids = None

        not_in_owner_ids = filter_params.get_as_object('not_in_owner_ids')
        if isinstance(not_in_owner_ids, str):
            not_in_owner_ids = not_in_owner_ids.split(',')
        if not isinstance(not_in_owner_ids, list):
            not_in_owner_ids = None

        def predicate(item: GroupV1) -> bool:
            if id and item.id != id:
                return False
            if ids and item.id not in ids:
                return False
            if not_in_owner_ids and item.owner_id in not_in_owner_ids:
                return False
            if owner_id and item.owner_id != owner_id:
                return False
            if title and item.title != title:
                return False
            if group_active is not None and item.group_active != group_active:
                return False
            if active_since and item.active_since != active_since:
                return False
            return True

        return predicate

    def get_page_by_filter(self, context: Optional[IContext], filter_params: FilterParams,
                           paging: PagingParams) -> DataPage:
        return super().get_page_by_filter(
            context,
            self.__compose_filter(filter_params),
            paging,
            None,
            None
        )

    def delete_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        return super().delete_by_filter(context, self.__compose_filter(filter_params))



# -*- coding: utf-8 -*-
from datetime import datetime
import json
import bottle
import asyncio

from typing import List
from pip_services4_components.refer import Descriptor, IReferences
from pip_services4_http.controller import RestOperations, RestController
from pip_services4_commons.errors import BadRequestException
from pip_services4_data.query import FilterParams
from pip_services4_components.context import Context

from eic_aichat_users.groupmemberships.data.GroupMembershipV1 import GroupMembershipV1
from eic_aichat_users.groupmemberships.logic.IGroupMembershipsService import IGroupMembershipsService
from eic_aichat_users.groups.logic.IGroupsService import IGroupsService
from eic_aichat_users.partialfacade.operations.version1.Authorize import AuthorizerV1
from eic_aichat_users.accounts.logic.IAccountsService import IAccountsService


class GroupsMembershipsOperation(RestOperations):
    def __init__(self):
        super().__init__()
        self._group_memberships: IGroupMembershipsService = None
        self._group: IGroupsService = None
        self._accounts_service: IAccountsService = None
        self._dependency_resolver.put('groupmemberships', Descriptor('aichatusers-groupmemberships', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put('groups', Descriptor('aichatusers-groups', 'service', '*', '*', '1.0'))
        self._dependency_resolver.put("accounts-service", Descriptor('aichatusers-accounts', 'service', '*', '*', '1.0'))

    def set_references(self, references: IReferences):
        super().set_references(references)
        self._accounts_service = self._dependency_resolver.get_one_required('accounts-service')
        self._group_memberships = self._dependency_resolver.get_one_required('groupmemberships')
        self._group = self._dependency_resolver.get_one_required('groups')

    def get_memberships(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id

        filter_params = self._get_filter_params()
        paging = self._get_paging_params()

        try:
            page = self._group_memberships.get_memberships(context, filter_params, paging)

            ids = []
            for item in page.data:
                ids.append(item.id)

            userMap = self._accounts_service.get_map_by_ids(context, ids)
            groupMap = self._group.get_map_by_ids(context, ids)

            for item in page.data:
                item.group_name = groupMap[item.group_id].title
                item.profile_name = userMap[item.profile_id].name
                item.profile_email = userMap[item.profile_id].login

            return self._send_result(page)
        except Exception as err:
            return self._send_error(err)

    def get_membership_by_id(self, membership_id: str):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            membership = self._group_memberships.get_membership_by_id(context, membership_id)
            if membership is None or membership.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_ID", "Id does not exist")
            
            user = self._accounts_service.get_account_by_id(context, membership.profile_id)
            group = self._group.get_group_by_id(context, membership.group_id)

            membership.group_name = group.title
            membership.profile_name = user.name
            membership.profile_email = user.login
            return self._send_result(membership)
        except Exception as err:
            return self._send_error(err)

    def create_membership(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id

        data = bottle.request.json
        groupMembership_data = data if isinstance(data, dict) else json.loads(data or '{}')
        groupMembership = GroupMembershipV1(**groupMembership_data)

        try:
            group = self._group.get_group_by_id(context, groupMembership.group_id)
            if group is None or group.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_GROUP_ID", "Group id does not exist")
            if group.owner_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "User id does not match group owner id")
            
            user = self._accounts_service.get_account_by_id(context, groupMembership.profile_id)
            if user is None or user.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_USER_ID", "User id does not exist")

            group.member_count += 1
            self._group.update_group(context, group)

            memberships = self._group_memberships.create_membership(context, groupMembership)

            memberships.group_name = group.title
            memberships.profile_name = user.name
            memberships.profile_email = user.login
            return self._send_result(memberships)
        except Exception as err:
            return self._send_error(err)
        
    def create_memberships_bulk(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id

        data = bottle.request.json
        memberships_data = data if isinstance(data, list) else json.loads(data or '[]')

        if not isinstance(memberships_data, list):
            raise BadRequestException(self._get_trace_id(), "INVALID_INPUT", "Expected a list of memberships")

        results = []
        try:
            for membership_dict in memberships_data:
                groupMembership = GroupMembershipV1(**membership_dict)

                group = self._group.get_group_by_id(context, groupMembership.group_id)
                if group is None or group.id == "":
                    raise BadRequestException(self._get_trace_id(), "WRONG_GROUP_ID", "Group id does not exist")
                if group.owner_id != user_id:
                    raise BadRequestException(self._get_trace_id(), "WRONG_USER", "User id does not match group owner id")
                
                user = self._accounts_service.get_account_by_id(context, groupMembership.profile_id)
                if user is None or user.id == "":
                    raise BadRequestException(self._get_trace_id(), "WRONG_USER_ID", "User id does not exist")
                
                group.member_count += 1
                self._group.update_group(context, group)
                
                created_membership = self._group_memberships.create_membership(context, groupMembership)

                created_membership.group_name = group.title
                created_membership.profile_name = user.name
                created_membership.profile_email = user.login
                results.append(created_membership)

            return self._send_result(results)
        except Exception as err:
            return self._send_error(err)
        
    def update_membership(self):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
    
        data = bottle.request.json
        groupMembership_data = data if isinstance(data, dict) else json.loads(data or '{}')
        groupMembership = GroupMembershipV1(**groupMembership_data)

        try:
            group = self._group.get_group_by_id(context, groupMembership.group_id)
            if group is None or group.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_GROUP_ID", "Group id does not exist")
            if group.owner_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "User id does not match group owner id")
            
            user = self._accounts_service.get_account_by_id(context, groupMembership.profile_id)
            if user is None or user.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_USER_ID", "User id does not exist")
            
            
            memberships = self._group_memberships.update_membership(context, groupMembership)

            memberships.group_name = group.title
            memberships.profile_name = user.name
            memberships.profile_email = user.login
            return self._send_result(memberships)
        except Exception as err:
            return self._send_error(err)
        
    def delete_membership_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            membership = self._group_memberships.get_membership_by_id(context, id)
            group = self._group.get_group_by_id(context, membership.group_id)
            if membership is None or membership.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_MEMBERSHIP_ID", "Membership id does not exist")
            if membership.profile_id != user_id and group.owner_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "Profile id does not match user id")
            
            group.member_count -= 1
            self._group.update_group(context, group)
            
            memberships = self._group_memberships.delete_membership_by_id(context, id)
            return self._send_result(memberships)
        except Exception as err:
            return self._send_error(err)
        
    def delete_membership_by_filter(self):
        context = Context.from_trace_id(self._get_trace_id())
        filter_params = self._get_filter_params()
        try:
            res = self._group.delete_groups_by_filter(context, filter_params)
            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def activate_membership_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            membership = self._group_memberships.get_membership_by_id(context, id)
            if membership is None or membership.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_ID", "Id does not exist")
            
            group = self._group.get_group_by_id(context, membership.group_id)
            if group is None or group.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_GROUP_ID", "Group id does not exist")
            if group.owner_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "User id does not match group owner id")
            
            membership.active = True
            membership.member_since = datetime.utcnow()
            
            res = self._group_memberships.update_membership(context, membership)

            user = self._accounts_service.get_account_by_id(context, membership.profile_id)
            res.group_name = group.title
            res.profile_name = user.name
            res.profile_email = user.login

            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def deactivate_membership_by_id(self, id):
        context = Context.from_trace_id(self._get_trace_id())
        user_id = bottle.request.user_id
        try:
            membership = self._group_memberships.get_membership_by_id(context, id)
            if membership is None or membership.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_ID", "Id does not exist")
            
            group = self._group.get_group_by_id(context, membership.group_id)
            if group is None or group.id == "":
                raise BadRequestException(self._get_trace_id(), "WRONG_GROUP_ID", "Group id does not exist")
            if group.owner_id != user_id:
                raise BadRequestException(self._get_trace_id(), "WRONG_USER", "User id does not match group owner id")
            
            membership.active = False
            
            res = self._group_memberships.update_membership(context, membership)

            user = self._accounts_service.get_account_by_id(context, membership.profile_id)
            res.group_name = group.title
            res.profile_name = user.name
            res.profile_email = user.login

            return self._send_result(res)
        except Exception as err:
            return self._send_error(err)
        
    def register_routes(self, controller: RestController, auth: AuthorizerV1):
            controller.register_route_with_auth('get', '/users/groups/memberships', None, auth.signed(), lambda: self.get_memberships())
            
            controller.register_route_with_auth('get', '/users/groups/:id/memberships', None, auth.signed(), lambda id: self.get_membership_by_id(id))

            controller.register_route_with_auth('post', '/users/groups/memberships', None, auth.signed(), lambda: self.create_membership())

            controller.register_route_with_auth('post', '/users/groups/memberships/bulk', None, auth.signed(), lambda: self.create_memberships_bulk())

            controller.register_route_with_auth('put', '/users/groups/memberships', None, auth.signed(), lambda: self.update_membership())

            controller.register_route_with_auth('delete', '/users/groups/:id/memberships', None, auth.signed(), lambda id: self.delete_membership_by_id(id))

            controller.register_route_with_auth('delete', '/users/groups/memberships', None, auth.admin(), lambda: self.delete_membership_by_filter())

            controller.register_route_with_auth('post', '/users/groups/:id/memberships/activate', None, auth.signed(), lambda id: self.activate_membership_by_id(id))

            controller.register_route_with_auth('post', '/users/groups/:id/memberships/deactivate', None, auth.signed(), lambda id: self.deactivate_membership_by_id(id))
            
        



# -*- coding: utf-8 -*-

__all__ = [
    'IRolesPersistence', 'RolesMemoryPersistence', 'RolesMongoDbPersistence'
]

from .IRolesPersistence import IRolesPersistence
from .RolesMemoryPersistence import RolesMemoryPersistence
from .RolesMongoDbPersistence import RolesMongoDbPersistence



# -*- coding: utf-8 -*-
from datetime import datetime
import re
from typing import Dict, List, Optional

from pip_services4_components.config import ConfigParams, IConfigurable
from pip_services4_components.context import IContext
from pip_services4_components.refer import IReferenceable, IReferences, Descriptor
from pip_services4_data.keys import IdGenerator
from pip_services4_data.query import DataPage, FilterParams, PagingParams
from pip_services4_observability.log import CompositeLogger
from pip_services4_commons.errors import BadRequestException, NotFoundException

from ..data import AccountV1
from ..data.AccountActivityTypeV1 import AccountActivityTypeV1
from ..persistence import IAccountsPersistence
from ..logic.IAccountsService import IAccountsService

# # Optional dependency
# try:
#     from client_activities_python import IActivitiesClientV1, PartyActivityV1
# except ImportError:
#     IActivitiesClientV1 = None
#     PartyActivityV1 = None


class AccountsService(IAccountsService, IConfigurable, IReferenceable):
    _persistence: IAccountsPersistence = None
    # _activities_client: Optional[IActivitiesClientV1] = None
    _logger: CompositeLogger = CompositeLogger()
    _login_as_email: bool = False

    _email_regex = re.compile(r'^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@'
                              r'((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$')

    def configure(self, config: ConfigParams):
        self._login_as_email = config.get_as_boolean_with_default('options.login_as_email', self._login_as_email)

    def set_references(self, references: IReferences):
        self._logger.set_references(references)
        self._persistence = references.get_one_required(Descriptor('aichatusers-accounts', 'persistence', '*', '*', '1.0'))
        # self._activities_client = references.get_one_optional(Descriptor('aichatactivities', 'client', '*', '*', '1.0'))

    def get_accounts(self, context: Optional[IContext], filter_params: FilterParams, paging: PagingParams) -> DataPage:
        return self._persistence.get_page_by_filter(context, filter_params, paging)

    def get_account_by_id(self, context: Optional[IContext], account_id: str) -> AccountV1:
        return self._persistence.get_one_by_id(context, account_id)

    def get_account_by_login(self, context: Optional[IContext], login: str) -> AccountV1:
        return self._persistence.get_one_by_login(context, login)

    def get_account_by_id_or_login(self, context: Optional[IContext], id_or_login: str) -> AccountV1:
        return self._persistence.get_one_by_id_or_login(context, id_or_login)

    def create_account(self, context: Optional[IContext], account: AccountV1) -> AccountV1:
        self._validate_account(context, account)

        existing = self._persistence.get_one_by_login(context, account.login)
        if existing:
            raise BadRequestException(context, 'ALREADY_EXIST', f'User account {account.login} already exists') \
                .with_details('login', account.login)

        account.id = account.id or IdGenerator.next_long()
        account.active = account.active if account.active is not None else True
        account.create_time = account.create_time or datetime.now()

        created = self._persistence.create(context, account)
        self._log_user_activity(context, created, AccountActivityTypeV1.AccountCreated)
        return created

    def update_account(self, context: Optional[IContext], account: AccountV1) -> AccountV1:
        self._validate_account(context, account)

        data = self._persistence.get_one_by_login(context, account.login)
        if data and data.id != account.id:
            raise BadRequestException(context, 'ALREADY_EXIST', f'Login {account.login} is already in use') \
                .with_details('login', account.login)

        old = self._persistence.get_one_by_id(context, account.id)
        if not old:
            raise NotFoundException(context, 'NOT_FOUND', f'Account {account.id} was not found') \
                .with_details('id', account.id)

        updated = self._persistence.update(context, account)
        self._log_user_activity(context, updated, AccountActivityTypeV1.AccountChanged)
        return updated

    def delete_account_by_id(self, context: Optional[IContext], account_id: str) -> AccountV1:
        account = self._persistence.get_one_by_id(context, account_id)
        if not account:
            return None

        account.deleted = True
        deleted = self._persistence.update(context, account)

        self._log_user_activity(context, deleted, AccountActivityTypeV1.AccountDeleted)
        return deleted

    def drop_account_by_id(self, context: Optional[IContext], account_id: str) -> AccountV1:
        account = self._persistence.delete_by_id(context, account_id)
        if account:
            self._log_user_activity(context, account, AccountActivityTypeV1.AccountDropped)
        return account

    def _validate_account(self, context: Optional[IContext], account: AccountV1):
        if not account.name:
            raise BadRequestException(context, 'NO_NAME', 'Missing account name')

        if self._login_as_email:
            if not account.login:
                raise BadRequestException(context, 'NO_EMAIL', 'Missing account email')
            if not self._email_regex.match(account.login):
                raise BadRequestException(context, 'WRONG_EMAIL', f'Invalid email: {account.login}') \
                    .with_details('login', account.login)
        else:
            if not account.login:
                raise BadRequestException(context, 'NO_LOGIN', 'Missing account login')

    def _log_user_activity(self, context: Optional[IContext], account: AccountV1, activity_type: str):
        pass
        # if self._activities_client and PartyActivityV1:
        #     try:
        #         activity = PartyActivityV1(
        #             None,
        #             activity_type,
        #             {'id': account.id, 'type': 'account', 'name': account.name}
        #         )
        #         self._activities_client.log_party_activity(context, activity)
        #     except Exception as err:
        #         self._logger.error(context, err, "Failed to log user activity")

    def get_map_by_ids(self, context: Optional[IContext], account_ids: List[str]) -> Dict[str, AccountV1]:
        accounts = self._persistence.get_list_by_ids(context, account_ids)
        result: Dict[str, AccountV1] = {account.id: account for account in accounts if account.id is not None}
        return result



# -*- coding: utf-8 -*-
from datetime import datetime
from typing import Optional

from pip_services4_components.config import ConfigParams, IConfigurable
from pip_services4_components.context import IContext
from pip_services4_components.refer import IReferenceable, IReferences, Descriptor
from pip_services4_data.keys import IdGenerator
from pip_services4_data.query import FilterParams, PagingParams, DataPage
from pip_services4_rpc.commands import ICommandable, CommandSet

from eic_aichat_users.groupmemberships.logic.IGroupMembershipsService import IGroupMembershipsService

from ..data import GroupMembershipV1
from ..persistence import IGroupMembershipsPersistence


class GroupMembershipsService(IGroupMembershipsService, IConfigurable, IReferenceable, ICommandable):
    _persistence: IGroupMembershipsPersistence = None

    def configure(self, config: ConfigParams):
        pass

    def set_references(self, references: IReferences):
        self._persistence = references.get_one_required(
            Descriptor('aichatusers-groupmemberships', 'persistence', '*', '*', '1.0')
        )

    def get_memberships(self, context: Optional[IContext], filter_params: FilterParams,
                              paging: PagingParams) -> DataPage:
        return self._persistence.get_page_by_filter(context, filter_params, paging)

    def get_membership_by_id(self, context: Optional[IContext], membership_id: str) -> Optional[GroupMembershipV1]:
        return self._persistence.get_one_by_id(context, membership_id)

    def create_membership(self, context: Optional[IContext], membership: GroupMembershipV1) -> GroupMembershipV1:
        membership.id = membership.id or IdGenerator.next_long()
        membership.created = membership.created or datetime.utcnow()
        return self._persistence.create(context, membership)

    def update_membership(self, context: Optional[IContext], membership: GroupMembershipV1) -> GroupMembershipV1:
        return self._persistence.update(context, membership)

    def delete_membership_by_id(self, context: Optional[IContext], membership_id: str) -> Optional[GroupMembershipV1]:
        return self._persistence.delete_by_id(context, membership_id)

    def delete_memberships_by_filter(self, context: Optional[IContext], filter_params: FilterParams) -> None:
        self._persistence.delete_by_filter(context, filter_params)



# -*- coding: utf-8 -*-
from abc import ABC, abstractmethod
from typing import Optional

from pip_services4_components.context import IContext
from pip_services4_data.query import FilterParams, PagingParams, DataPage

from ..data import PartyActivityV1


class IActivitiesPersistence(ABC):
    """
    Interface for party activities persistence.
    """
    
    @abstractmethod
    def get_page_by_filter(self, context: IContext, filter_params: FilterParams, paging_params: PagingParams) -> DataPage:
        """
        Gets a page of party activities by specified filter
        
        Args:
            context: Operation context
            filter_params: Filter parameters
            paging_params: Paging parameters
            
        Returns:
            DataPage with retrieved party activities
        """
        pass

    @abstractmethod
    def get_one_by_id(self, context: IContext, id: str) -> Optional[PartyActivityV1]:
        """
        Gets party activity by id
        
        Args:
            context: Operation context
            id: Activity id
            
        Returns:
            Retrieved party activity
        """
        pass

    @abstractmethod
    def create(self, context: IContext, item: PartyActivityV1) -> PartyActivityV1:
        """
        Creates party activity
        
        Args:
            context: Operation context
            item: Party activity to create
            
        Returns:
            Created party activity
        """
        pass

    @abstractmethod
    def delete_by_id(self, context: IContext, id: str) -> Optional[PartyActivityV1]:
        """
        Deletes party activity by id
        
        Args:
            context: Operation context
            id: Activity id
            
        Returns:
            Deleted party activity
        """
        pass

    @abstractmethod
    def delete_by_filter(self, context: IContext, filter_params: FilterParams) -> None:
        """
        Deletes party activities by filter
        
        Args:
            context: Operation context
            filter_params: Filter parameters
        """
        pass 


# -*- coding: utf-8 -*-
from pip_services4_commons.convert import TypeCode
from pip_services4_data.validate import ObjectSchema


class AccountV1Schema(ObjectSchema):
    def __init__(self):
        super().__init__()

        self.with_optional_property('id', TypeCode.String)
        self.with_required_property('login', TypeCode.String)
        self.with_required_property('name', TypeCode.String)
        self.with_optional_property('create_time', TypeCode.DateTime)
        self.with_optional_property('deleted', TypeCode.Boolean)
        self.with_optional_property('active', TypeCode.Boolean)
        self.with_optional_property('about', TypeCode.String)
        self.with_optional_property('time_zone', TypeCode.String)
        self.with_optional_property('language', TypeCode.String)
        self.with_optional_property('theme', TypeCode.String)
        self.with_optional_property('custom_hdr', None)
        self.with_optional_property('custom_dat', None)
